///////////////////////////////////////////////////////////////////
//	depStream.cpp:	to read each line and find out included files//
//					add them to the file independence matrix	 //
//	ver 1.0														 //
//	Language:		Visual C++, ver 6.0							 //
//	Platform:													 //
//	Application:	CSE687 projet 1								 //
//	Author:			Qiang Zheng, qzheng@syr.edu					 //
///////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////
//	Purpose:	Each instance of this class is associated with a //
//				file whose name appears in the command line.	 //
//																 //
//				So when one file is opened, it will be scanned	 //
//				line by line to look for the names of the file	 //
//				being included and put those files into the		 //
//				dependence Matrix								 //
///////////////////////////////////////////////////////////////////

#include "iostream.h"
#include "depStream.h"

depMatrix depStream::dependence;

// constructor
depStream::depStream()
{
	fileName = NULL;
	index = -1;
}

depStream::depStream( const char* szName )
{
	fileName = NULL;
	index = -1;
	fileName = newStr( szName );
	ifstream( fileName );
	if ( !is_open() )
		cout<<"Error opening file "<<szName<<"\n";
	else
		index = dependence.indexOf( szName, true );
}

depStream::~depStream()
{
	if ( fileName )
		delete fileName;
}

void depStream::open( const char *s )
{
	index = -1;
	ifstream::open( s );
	if ( !is_open() )
		cout<<"Error opening file "<<s<<"\n";
	else
		index = dependence.indexOf( s,true );
}

void depStream::parseIt()
{
	if ( index == -1 )
		cout<<"Can't parse file "<<fileName<<"\n";
	char buffer[4096];
	char curFileName[1024];
	while( good() ) {
		getline( buffer, 4095, '\n' );
		int pos;
		if( foundPrefix( buffer, pos )
			&& getFileName( buffer, curFileName, pos ) ) {
			int curIndex = dependence.indexOf( curFileName, false );
			dependence.setDependence( this->index, curIndex );
		}
	}
}

bool depStream::foundPrefix( const char *s, int&pos )
{
	static const char*prefix="include";
	pos = 0;
	while ( s[pos] != '\0' ) {
		while ( s[pos] == ' ' ) pos++;
		while ( s[pos] == '\t' ) pos++;
		if ( s[pos] == '#' ) {
			pos++;
			if ( strncmp( prefix, &(s[pos]), 7 ) == 0 )	{
				pos += 7;
				return true;
			} else
				return false;
		} else
			return false;
	}
	return false;
}

bool depStream::getFileName( const char *s, char *out, int pos )
{
	while ( s[pos] != '\0' ) {
		while ( s[pos] == ' ' ) pos++;
		while ( s[pos] == '\t' ) pos++;
		if ( s[pos] == '\"' || s[pos] == '<' ) {
			char Delim;
			if ( s[pos] == '\"' ) Delim = '\"';
			else Delim = '>';
			pos++;
			int index = 0;
			while ( s[pos] != '\0' && s[pos] != '\"' && s[pos] != '>' )
				out[index++] = s[pos++];
			if ( s[pos] == '\0') return false;
			if ( s[pos] != Delim) return false;
			out[index] = '\0';
			return true;
		}else
			return false;
	}
	return false;
}